#
# Copyright (C) 2021-2022 by TeamYukki@Github, < https://github.com/TeamYukki >.
#
# This file is part of < https://github.com/TeamYukki/YukkiMusicBot > project,
# and is released under the "GNU v3.0 License Agreement".
# Please see < https://github.com/TeamYukki/YukkiMusicBot/blob/master/LICENSE >
#
# All rights reserved.

import os
import time
from datetime import datetime, timedelta
from typing import Union

from pyrogram.types import Voice

import config
from config import MUSIC_BOT_NAME
from YukkiMusic import app

from ..utils.formatters import (convert_bytes, get_readable_time,
                                seconds_to_min)

downloader = {}


class TeleAPI:
    def __init__(self):
        self.chars_limit = 4096
        self.sleep = config.TELEGRAM_DOWNLOAD_EDIT_SLEEP

    async def send_split_text(self, message, string):
        n = self.chars_limit
        out = [(string[i : i + n]) for i in range(0, len(string), n)]
        j = 0
        for x in out:
            if j <= 2:
                j += 1
                await message.reply_text(x)
        return True

    async def get_link(self, message):
        if message.chat.username:
            link = f"https://t.me/{message.chat.username}/{message.reply_to_message.message_id}"
        else:
            xf = str((message.chat.id))[4:]
            link = f"https://t.me/c/{xf}/{message.reply_to_message.message_id}"
        return link

    async def get_filename(
        self, file, audio: Union[bool, str] = None
    ):
        try:
            file_name = file.file_name
            if file_name is None:
                file_name = (
                    "Telegram Audio File"
                    if audio
                    else "Telegram Video File"
                )

        except:
            file_name = (
                "Telegram Audio File"
                if audio
                else "Telegram Video File"
            )
        return file_name

    async def get_duration(self, file):
        try:
            dur = seconds_to_min(file.duration)
        except:
            dur = "Unknown"
        return dur

    async def get_filepath(
        self,
        audio: Union[bool, str] = None,
        video: Union[bool, str] = None,
    ):
        if audio:
            try:
                file_name = (
                    audio.file_unique_id
                    + "."
                    + (
                        (audio.file_name.split(".")[-1])
                        if (not isinstance(audio, Voice))
                        else "ogg"
                    )
                )
            except:
                file_name = audio.file_unique_id + "." + ".ogg"
            file_name = os.path.join(
                os.path.realpath("downloads"), file_name
            )
        if video:
            try:
                file_name = (
                    video.file_unique_id
                    + "."
                    + (video.file_name.split(".")[-1])
                )
            except:
                file_name = video.file_unique_id + "." + "mp4"
            file_name = os.path.join(
                os.path.realpath("downloads"), file_name
            )
        return file_name

    async def download(self, _, message, mystic, fname):
        left_time = {}
        speed_counter = {}
        if os.path.exists(fname):
            return True

        async def progress(current, total):
            if current == total:
                return
            current_time = time.time()
            start_time = speed_counter.get(message.message_id)
            check_time = current_time - start_time
            if datetime.now() > left_time.get(message.message_id):
                percentage = current * 100 / total
                percentage = str(round(percentage, 2))
                speed = current / check_time
                eta = int((total - current) / speed)
                downloader[message.message_id] = eta
                eta = get_readable_time(eta)
                if not eta:
                    eta = "0 sec"
                total_size = convert_bytes(total)
                completed_size = convert_bytes(current)
                speed = convert_bytes(speed)
                text = f"""
**{MUSIC_BOT_NAME} Telegram Media Downloader**

**Total FileSize:** {total_size}
**Completed:** {completed_size} 
**Percentage:** {percentage[:5]}%

**Speed:** {speed}/s
**ETA:** {eta}"""
                try:
                    await mystic.edit(text)
                except:
                    pass
                left_time[
                    message.message_id
                ] = datetime.now() + timedelta(seconds=self.sleep)

        if len(downloader) > 3:
            timers = []
            for x in downloader:
                timers.append(downloader[x])
            low = min(timers)
            eta = get_readable_time(low)
            await mystic.edit_text(_["tg_1"].format(eta))
            return False

        speed_counter[message.message_id] = time.time()
        left_time[message.message_id] = datetime.now()
        try:
            X = await app.download_media(
                message.reply_to_message,
                file_name=fname,
                progress=progress,
            )
            try:
                downloader.pop(message.message_id)
            except:
                pass
            return True
        except:
            try:
                downloader.pop(message.message_id)
            except:
                pass
            await mystic.edit_text(_["tg_2"])
            return False
